@@ -269,7 +269,8 @@ def _favicon():
         b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
 
 
-def init_wsgi_proxy(resources: Dict[str, str] = {}) -> http_server.WSGIProxy:
+def init_wsgi_proxy(resources: Dict[str, str] = {}, session_factory=None) -> http_server.WSGIProxy:
+    set_session_factory(session_factory or LocalSessionFactory())
     proxy = http_server.WSGIProxy(res_conf=resources)
     filters = _get_filters()
     # filter configuration
