@@ -23,6 +23,7 @@ SOFTWARE.
 """
 
 
+import asyncio
 import html
 import simple_http_server.__utils as utils
 
@@ -50,10 +51,11 @@ class WSGIRequestHandler:
         self.response_headers = []
         self.body = []
 
-        self.server = routing_conf
+        self.routing_conf = routing_conf
         self.headers = self._parse_headers()
         self.query_parameters = utils.decode_query_string(self.query_string)
-        self.wfile = self
+        self.writer = self
+        self.reader = self
 
     @property
     def rfile(self):
@@ -69,6 +71,7 @@ class WSGIRequestHandler:
     @property
     def request_path(self):
         path = self.env['PATH_INFO']
+        path = path.encode('iso-8859-1').decode('utf-8')
         if path.startswith('/'):
             path = path[1:]
         return path
@@ -77,9 +80,35 @@ class WSGIRequestHandler:
     def query_string(self):
         return self.env['QUERY_STRING']
 
-    def handle(self) -> List[bytes]:
+    @property
+    def request(self):
+        return self
+
+    # Wrapper method for readline
+    async def readline(self):
+        if self.rfile:
+            return self.rfile.readline()
+        else:
+            return b""
+
+    async def read(self, n: int = -1):
+        if self.rfile:
+            return self.rfile.read(n)
+        else:
+            return b''
+
+    def write(self, data: bytes):
+        if isinstance(self.body, list):
+            self.body.append(data)
+        else:
+            self.body = [self.body, data]
+
+    def write_eof(self):
+        pass
+
+    async def handle_request(self) -> List[bytes]:
         handler = HTTPRequestHandler(self, environment=self.env)
-        handler.handle()
+        await handler.handle_request()
         self.start_response(self.status, self.response_headers)
         return self.body
 
@@ -105,7 +134,8 @@ class WSGIRequestHandler:
         for k, v in self.env.items():
             if k.startswith("HTTP_"):
                 header_name = k.replace("HTTP_", "")
-                header_name = "-".join([p[0].upper() + p[1:].lower() for p in header_name.split("_")])
+                header_name = "-".join([p[0].upper() + p[1:].lower()
+                                       for p in header_name.split("_")])
                 headers[header_name] = v
         return headers
 
@@ -159,7 +189,8 @@ class WSGIRequestHandler:
                          HTTPStatus.RESET_CONTENT,
                          HTTPStatus.NOT_MODIFIED)):
             try:
-                content: Any = self.server.error_page(code, html.escape(message, quote=False), html.escape(explain, quote=False))
+                content: Any = self.routing_conf.error_page(code, html.escape(
+                    message, quote=False), html.escape(explain, quote=False))
             except:
                 content: str = ""
             content_type, body = utils.decode_response_body_to_bytes(content)
@@ -173,9 +204,3 @@ class WSGIRequestHandler:
 
         if self.command != 'HEAD' and body:
             self.write(body)
-
-    def write(self, data: bytes):
-        if isinstance(self.body, list):
-            self.body.append(data)
-        else:
-            self.body = [self.body, data]
